/**************************************************************************

    tefax  -- front end to handle FAXes with efax or mgetty&sendfax
              and certain programms for view and print

    Copyright (C) 1997  Kai Uwe Tempel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    please send any bug reports, suggestions, greetings etc. to
	temu@server3.medizin.uni-leipzig.de

    or contact me in
	Kai Uwe Tempel, Gaertnerstr. 171, 04209 Leipzig, Germany

    call or fax: +49 341 4126014

***************************************************************************/
/***************************************************************************

tefax string processing functions

***************************************************************************/
/***************************************************************************

sp verarbeitet pfadnamen, kuerzt strings und 
konvertiert monatsnamen <-> num (int)

***************************************************************************/

/*LINTLIBRARY */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include "tefaxdef.h"
#include "debug.h"
#include "hcs.h"
#include "hlf.h"

/***************************************************************************/

/* einen string nach dem letzten slash abschneiden und einen zeiger
** auf die lokale static char zurueckgeben 
*/
char *pfnp(const char *c)             /* voller pfad: ...PATH/ */
{
    static char puf[BUFSIZ];

    CF(pfnp);
    strcpy(puf, c);
    *(strrchr(puf, '/') + 1) = '\0';
    CE;
    return puf;
}

/* einen string nach dem letzten slash abschneiden und einen zeiger
** zurueckgeben, der auf den mit malloc angeforderten bereich zeigt
*/
char *cpfnp(const char *c)            /* pfadname ...PATH */
{
  static char puf[BUFSIZ];
  
  CF(pfnn);
  strcpy(puf, c);
  *(strrchr(puf, '/') + 1) = '\0';
  
  CE;
  return strdup(puf);
}

/* einen string vor dem letzten slash abschneiden und einen zeiger
** auf die lokale static char zurueckgeben 
*/
char *pfnn(const char *c)             /* pfadname ...PATH */
{
    static char puf[BUFSIZ];

    CF(pfnn);
    strcpy(puf, c);
    *(strrchr(puf, '/')) = '\0';

    CE;
    return puf;
}

/* einen zeiger auf das erste zeichen nach dem letzten slash in einem 
** string zurueckgeben 
*/
char *pfnf(char *c)             /* filename */
{
  char *s;
  
  CF(pfnf);
  CE;
  s = strrchr(c, '/');
  if (s) 
    return s + 1;
  else 
    return c + strlen(c);
  
}

/* einen zeiger auf das erste zeichen nach dem letzten punkt in einem 
** string zurueckgeben 
*/
char *pfnx(char *c)             /* extension */
{
  char *s;
  
  CF(pfnx);
  CE;
  s = strrchr(c, '.');
  if (s) 
    return s + 1;
  else 
    return c + strlen(c);
  
}

/* das letzte zeichen eines strings zurueckgeben 
*/
int lastchar(const char *s)
{
  CF(lastchar);
  CE;
  return s[strlen(s) - 1];
}

/* bei strings, die laenger als i zeichen sind, werden das i-te und das zeichen 
** (i - 1) durch punkte ersetzt und der string an der stelle (i + 1) terminiert.
** zurueckgeben wird ein zeiger auf die lokale static char, so dass das original
** nicht veraendert wird 
*/
char *cutonend(char *s, int i)
{
  static char b[BUFSIZ];
  CF(cutonend);
  CE;
  if (!s) return NULL;
  if (strlen(s) <= i)
    return s;
  else {
    strcpy(b, s);
    if (i > BUFSIZ - 3) i = BUFSIZ - 3;
    strcpy(b + i - 2, "..");
    return b;
  }
}

/* bei strings, die laenger als i zeichen sind, werden die letzten (i - 2) zeichen
** mit zwei fuehrenden punkten zu einem string verbunden
** zurueckgeben wird ein zeiger auf die lokale static char, so dass das original
** nicht veraendert wird 
*/
char *cutonbeg(char *s, int i)
{
  static char b[BUFSIZ];
  int n;
  
  CF(cutonbeg);
  CE;
  if (!s) return NULL;
  else n = strlen(s);
  if (n <= i) 
    return s;
  else {
    strcpy(b, "..");
    return strcat(b, s + n + 2 - i);
  }
}

/* monthname wird zur initialisierung von monthtoi benoetigt, und muss daher
** sowohl die kleinschreibung, wie auch die grossschreibung der monatsnamen
** bereitstellen 
*/
char *monthname(int i)
{
  static char *name[] = {
    "...",
    "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
    "JAN", "FEB", "MAR", "APR", "MAY", "JUN", "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"
  };
  CF(monthname); CE;
  return (i < 1 || i > 24) ? name[0] : name[i];
}
 
/* monthtoi berechnet aus den ersten drei buchstaben der englischen monats-
** namen die quersumme der ascii-codes und gibt den integerwert des kalender-
** monats aus einer tabelle zurueck. auf diese weise werden zur umwandlung
** des monatsnamen in sein numerisches aequivalent ueberhaupt keine string-
** operationen benoetigt */
int monthtoi(const char *name)
{
  static int tabint[512];
  int i;
  static int inittab = 0; 

  CF(monthtoi);
  for(i = 0; i < 24 && inittab == 0; i++)
    tabint[monthname(i)[0] + monthname(i)[1] + monthname(i)[2]] = i % 12;
  inittab = 1;

  CE;
  return tabint[name[0] + name[1] + name[2]];
}

/* im prinzip das original von k&r, nur dass das zeilenendezeichen entfernt
** wird. die gelesenen zeichen werden in s gespeichert. der platz dafuer
** muss von der aufrufenden funktion bereitgestellt und die groesse in lim
** uebergeben werden */
int getline(char *s, int lim, FILE *f)
{
    int c = 0,i;

    CF(getline);
    for (i = 0; i < lim - 1 && (c = fgetc(f)) != EOF && c != '\n'; i++)
        s[i] = c;
    s[i] = '\0';
    CE;
    return (c == EOF) ? c : i;
}

/* um nicht die ausgaben mit ncurses auf dem bildschirm ausrichten zu muessen
** werden vor einem string so viele leerzeichen eingefuegt, dass er auf 80 
** zeichen zentriert erscheint 
*/
char *fillforcenter(char *s)
{
  static char c[BUFSIZ];
  int i;
  
  CF(fillforcenter);
  if (strlen(s) > 78) s = "error in fillforcenter! input too long";
                                /* ein string mit 40 leerzeichen */
  strcpy(c, "\n                                        ");
  i = (80 - strlen(s))/2;
  if (i < 1) i = 1;
  strcpy(c + i, s);
  CE;
  return c;
}

/* shellex ist ein ersatz fuer die bibliotheksfunktion system(), damit
** die die systemaufrufe in der datei lsc (log system calls) dokumentiert
** werden koennen. wenn die ausfuehrung der externen programme mit der
** kommandozeilenoption -n unterbunden wurde, werden die aufrufe nicht 
** an die shell weitergereicht, aber zur fehlersuche trotzdem mitge-
** schrieben. diese eintraege sind dann als created gekennzeichnet
** wenn die datei nicht geoeffnet werden kann, werden die systemauf-
** rufe auch nicht ausgefuehrt. damit ist sichergestellt, dass alle
** abgehenden faxe mit empfaenger und gesendeten dateien dokumentiert
** werden 
*/
int shellex(char *s)	      
{
  FILE *lsc = NULL;
  int i = -1;
  char t[50] = "";
  struct tm *tp;
  time_t tt;

  CF(shellex);
  if (!s) { CE; return i; }

  if (time(&tt) > 0) {		/* die zeit */
    tp = localtime(&tt);
    strftime(t, 20, "%d.%m.%H:%M", tp);
  }

  if ((lsc = fopen(S_(fl_sysc_lg), "a"))) {
    if (O_GET(ARG_n)) {
      fprintf(lsc, "at %s created: %s\n", t, s);
      i = 0;
    }
    else {
      fprintf(lsc, "at %s syscall: %s\n", t, s);
      i = system(s);
    }
    fclose(lsc);
  }
  CE;
  return i;
}

/* shellxx arbeitet wie shellex, jedoch werden die aufrufe auch
** dann ausgefuehrt, wenn die logdatei nicht geoeffnet werden konnte,
** weil zum beispiel beim rekursiven start von tefax unter X11 der
** name der logdatei noch gar nicht bekannt ist 
*/
int shellxx(char *s)
{
  FILE *lsc = NULL;
  char t[50] = "";
  struct tm *tp;
  time_t tt;

  CF(shellxx);
  if (!s) { CE; return -1; }

  if (time(&tt) > 0) {		/* die zeit */
    tp = localtime(&tt);
    strftime(t, 20, "%d.%m.%H:%M", tp);
  }

  if ((lsc = fopen(S_(fl_sysc_lg), "a"))) {
    fprintf(lsc, "at %s syscall: %s\n", t, s);
    fclose(lsc);
  }
  CE;
  return system(s);
}

/* gl ist eine abwandlung von getline speziell fuer seeincoming um die
** mails von efax aus der standardeingabe zu lesen 
*/
int gl(char *s, int lim)
{
    int c = 0,i;

    CF(gl);

    for (i = 0; i<lim-1 && (c = getchar()) != EOF && c != '\n'; i++)
        s[i] = c;
    s[i] = '\0';
    CE; return i;
}

/* die explizite deklaration von umask ist notwendig, weil sich sonst
** der compiler ueber eine implizite deklaration von umask beschwert
** laut man-page soll diese funktion zwar in der unistd.h deklariert
** sein, war aber bei mir nicht der fall. da ich die deklaration aber
** ueberhaupt nicht gefunden habe und es in dieser form funktioniert
** habe ich es so gelassen 
*/
int umask(int mask);

/* seeincoming liest die mails von efax mit gl aus der standardeingabe 
** und schreibt die relevanten daten in eine datei, deren namen mit 'z'
** beginnt und folgendes format hat: zAUFLOESUNG-FAXNAME.ABSENDER wobei
** die aufloesung durch die buchstaben 'n' oder 'f' beschrieben wird 
*/
void seeincoming()
{
    int len = 0, reclines = 0;
    char line[MAXLINE] = "";
    char s1[MAXLINE]   = "";
    char s2[MAXLINE]   = "";
    char d1[MAXLINE]   = "";
    char d2[MAXLINE]   = "";
    char contr[BUFSIZ] = "";
    char contc[BUFSIZ] = "";
    char contf[BUFSIZ] = "";    
    char messf[BUFSIZ] = "";

    char r = 'z';
    char *s, *c;
    FILE *f;

    CF(seeincoming);

    while ((len = gl(line, MAXLINE)) > 0) {
      /* die erste datei eines faxes:
      ** efax: received -> 28Jan203830.001 (39:52) 
      */
	if (strncmp(line, S_(inf_recv), strlen(S_(inf_recv))) == 0) {
	    if (line[15] == '-' && line[16] == '>') {
		c = line + 18;
		s = s1;
		while ((*s++ = *c++) != ' ');
		*--s ='\0';
		sprintf(contf, "%s %s", S_(inf_file), s1);	/* first faxfile is ... */
	    }
	    /* die zeile mit der zeilenanzahl beginnt ebenso:
	    ** efax: received 2264 lines, 0 errors
	    */
		/* may be we can found res in number of lines */
	    else if (sscanf(line, "%s %s %d", d1, d2, &reclines) == 3) {
	      if (reclines > 900 && reclines < 1800 && r == 'z') {
		r = 'n';
		sprintf(contr, "%d %s", reclines, S_(inf_nres)); /* .... think it's normal res  */
	      }
	      if (reclines >=1800 && reclines < 3600 && r == 'z') {
		r = 'f';
		sprintf(contr, "%d %s", reclines, S_(inf_fres)); /* ... think it's fine res */
	      }
	    }
	}
	/* die gegenstelle wird mit FTSI abgefragt:
	** efax: [+FTSI: "     HEWLETT PACKARD"<CR><LF>]
	*/
	if (strncmp(line, S_(inf_ftsi), strlen(S_(inf_ftsi))) == 0) {
	    c = line + strlen(S_(inf_ftsi));
	    s = s2;

	    /* eventuell muessen leerzeichen am anfang oder ende entfernt werden */
	    while (isspace(*c) || *c == '"') c++;
	    while ((*s++ = *c++) != '"');
 	    *--s ='\0';
	    while (isspace(*--s)) *--s ='\0';
	    sprintf(contc, "%s %s", S_(inf_came), s2); /* fax cames from .... */
	}
	/* im faxscript "grep session\ 98lpi", besser ist das ergebnis von +FDCS= VR,..... 
	** efax: response "+FDCS: 1,3,0,2,0,0,0,5" 
	*/
	if (strncmp(line, S_(inf_fdcs), strlen(S_(inf_fdcs))) == 0) {
	  for(c = line; !isdigit(*c); c++);
	  if (*c == '0') {
	    r = 'n';
	    sprintf(contr, "%s --> 98 lpi\n", line); /* it's normal res  */
	  }
	  if (*c == '1') {
	    r = 'f';
	    sprintf(contr, "%s --> 196 lpi\n", line); /* it's fine res  */
	  }
	}
    }

    /* daten abspeichern */
    if (strrchr(s1,'.')) {
	s = strrchr(s1, '.');
	*++s = '\0';

	/* dateinamen zusammensetzen */
	sprintf(line, "%sz%c-%s%s", S_(pth_ic), r, s1, s2);

	/* zugriffsrechte fuer alle user setzen */
	len = umask(0000);
	if ((f = fopen(line, "w"))) {
	  fprintf(f, "%s\n%s\n%s\n", contc, contf, contr);
	  fclose(f);

	  /* damit die datei nicht ganz leer ist, werden zumindest die gewonnen daten
	  ** eingtragen, und man kann auch mal 'von hand' nachschauen ... */
	  sprintf(messf, "echo %s %s\n", S_(sts_wrote), line); /* wrote file ... */
	  shellex(messf);
	}
	umask(len);
    }
    CE;
}






